/******************************************************************************
 * FileName:        main.c
 * Processor:       STM32F4XX
 * Versions:        0.85
 *
 * Author:          Arkadiusz Prus        10.02.2013
 ******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"
#include "stm32f4_discovery.h"
#include "stm32f4_discovery_lcd.h"
#include "main.h"
#include "systick.h"
#include "snake.h"
#include "debug.h"
#include "stmpe811qtr.h"
#include "ff.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

#define OPTIONS_START_POS_X	200
#define OPTIONS_END_POS_X 	320
#define OPTIONS_START_POS_Y 140
#define OPTIONS_END_POS_Y 	200

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint8_t capture_Flag = ENABLE;
FATFS Fatfs; /* File system object */

enum
{
	INIT,
	WAIT_PLAY,
	OPTIONS,
	INIT_PLAY,
	PLAY,
};

/* Private function prototypes -----------------------------------------------*/
static bool options_position(TS_STATE *pstate);

/* Private functions ---------------------------------------------------------*/
/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
  TS_STATE *pstate = 0;    // touch screen state
  uint8_t state = INIT, old_state = 0xFF;

  uart_init();
  /*!< At this stage the microcontroller clock setting is already configured, 
       this is done through SystemInit() function which is called from startup
       file (startup_stm32f4xx.s) before to branch to application main.
       To reconfigure the default setting of SystemInit() function, refer to
       system_stm32f4xx.c file
     */
  /* SysTick end of count event each 1ms */
  SysTick_Init();

  /* Touch screen initial config */
  IOE_Config();

  /* Initialize the LCD */
  STM32f4_Discovery_LCD_Init();
  LCD_Clear(LCD_COLOR_BLACK);

  f_mount(0, &Fatfs); /* Register volume work area (never fails) */

  while(true)
  {
	  if(old_state != state)
	  {
		  my_printf("state %d\r\n", state);
		  old_state = state;
	  }

	  switch (state)
	  {
	  case INIT:
		  LCD_DispPic_SD(0, 0, 320, 240, "SNAKE.DAT");
		  state = WAIT_PLAY;
		  break;

	  case WAIT_PLAY:
	      pstate = IOE_TS_GetState();
	      delay_ms(20);
	      if (pstate->TouchDetected)
	      {
	    	  my_printf("Touch x=%d, y=%d\r\n", pstate->X, pstate->Y);
	    	  if(options_position(pstate))
	    		  state = OPTIONS;
	    	  else
	    		  state = INIT_PLAY;
	      }
		  break;

	  case OPTIONS:
		  snakeOptions();
		  state = INIT;
		  do
		  {
			  pstate = IOE_TS_GetState();
		  } while(!pstate->TouchDetected);
		  break;

	  case INIT_PLAY:
		  printFrame();
		  initSnake();
		  state = PLAY;
		  break;

	  case PLAY:
		  if (false == snakeTask())
		  {
			  state = INIT;
			  do
			  {
				  pstate = IOE_TS_GetState();
			  } while(!pstate->TouchDetected);
			  break;
		  }
		  pstate = IOE_TS_GetState();
		  if(pstate->TouchDetected)
		  {
			  calcDirection(pstate->X, pstate->Y);
		  }
		  break;
	  default:
		  state = INIT;
		  break;
	  }
  }

  return 0;
}

static bool options_position(TS_STATE *pstate)
{
	bool res = false;

	if ((pstate->X >= OPTIONS_START_POS_X) && (pstate->X <= OPTIONS_END_POS_X) &&
		(pstate->Y >= OPTIONS_START_POS_Y) && (pstate->Y <= OPTIONS_END_POS_Y) )
	{
		res = true;
	}

	return res;
}


/**
  * @}
  */


/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
